@@ -1,5 +1,5 @@
 // vim:expandtab:shiftwidth=2:tabstop=2:
-// Copyright (C) 2013 Canonical Ltd.
+// Copyright (C) 2013-2015 Canonical Ltd.
 
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Lesser General Public
@@ -18,6 +18,8 @@
 #include "oxideqquickwebcontext_p.h"
 #include "oxideqquickwebcontext_p_p.h"
 
+#include <limits>
+
 #include <QMutex>
 #include <QMutexLocker>
 #include <QQmlEngine>
@@ -27,16 +29,10 @@
 #include <QThread>
 #include <QtQuickVersion>
 #include <QWeakPointer>
-#if defined(ENABLE_COMPOSITING)
-#if QT_VERSION < QT_VERSION_CHECK(5, 3, 0)
-#include <QtQuick/private/qsgcontext_p.h>
-#else
-#include <QtGui/private/qopenglcontext_p.h>
-#endif
-#endif
 
 #include "qt/core/api/oxideqnetworkcallbackevents.h"
 #include "qt/core/api/oxideqstoragepermissionrequest.h"
+#include "qt/quick/oxide_qquick_init.h"
 
 #include "oxideqquickcookiemanager_p.h"
 #include "oxideqquickuserscript_p.h"
@@ -48,14 +44,6 @@
 
 namespace {
 
-OxideQQuickWebContext* g_default_context;
-
-void DestroyDefaultContext() {
-  OxideQQuickWebContext* context = g_default_context;
-  g_default_context = NULL;
-  delete context;
-}
-
 QVariant networkCookiesToVariant(const QList<QNetworkCookie>& cookies) {
   QList<QVariant> list;
   Q_FOREACH(QNetworkCookie cookie, cookies) {
@@ -86,16 +74,17 @@ namespace qquick {
 
 using namespace webcontextdelegateworker;
 
-class WebContextIODelegate : public oxide::qt::WebContextAdapter::IODelegate {
+class WebContextIODelegate : public oxide::qt::WebContextProxyClient::IOClient {
  public:
   WebContextIODelegate() {}
   virtual ~WebContextIODelegate() {}
 
-  void OnBeforeURLRequest(OxideQBeforeURLRequestEvent* event) Q_DECL_FINAL;
-  void OnBeforeSendHeaders(OxideQBeforeSendHeadersEvent* event) Q_DECL_FINAL;
+  void OnBeforeURLRequest(OxideQBeforeURLRequestEvent* event) override;
+  void OnBeforeRedirect(OxideQBeforeRedirectEvent* event) override;
+  void OnBeforeSendHeaders(OxideQBeforeSendHeadersEvent* event) override;
   void HandleStoragePermissionRequest(
-      OxideQStoragePermissionRequest* req) Q_DECL_FINAL;
-  bool GetUserAgentOverride(const QUrl& url, QString* user_agent) Q_DECL_FINAL;
+      OxideQStoragePermissionRequest* req) override;
+  bool GetUserAgentOverride(const QUrl& url, QString* user_agent) override;
 
   QMutex lock;
 
@@ -112,11 +101,10 @@ void WebContextIODelegate::OnBeforeURLRequest(
     delegate = network_request_delegate.toStrongRef();
   }
   if (!delegate) {
-    delete event;
     return;
   }
 
-  emit delegate->callEntryPointInWorker("onBeforeURLRequest", event);
+  delegate->CallEntryPointInWorker("onBeforeURLRequest", event);
 }
 
 void WebContextIODelegate::OnBeforeSendHeaders(
@@ -127,11 +115,24 @@ void WebContextIODelegate::OnBeforeSendHeaders(
     delegate = network_request_delegate.toStrongRef();
   }
   if (!delegate) {
-    delete event;
     return;
   }
 
-  emit delegate->callEntryPointInWorker("onBeforeSendHeaders", event);
+  delegate->CallEntryPointInWorker("onBeforeSendHeaders", event);
+}
+
+void WebContextIODelegate::OnBeforeRedirect(
+    OxideQBeforeRedirectEvent* event) {
+  QSharedPointer<IOThreadController> delegate;
+  {
+    QMutexLocker locker(&lock);
+    delegate = network_request_delegate.toStrongRef();
+  }
+  if (!delegate) {
+    return;
+  }
+
+  emit delegate->CallEntryPointInWorker("onBeforeRedirect", event);
 }
 
 void WebContextIODelegate::HandleStoragePermissionRequest(
@@ -142,11 +143,10 @@ void WebContextIODelegate::HandleStoragePermissionRequest(
     delegate = storage_access_permission_delegate.toStrongRef();
   }
   if (!delegate) {
-    delete req;
     return;
   }
 
-  emit delegate->callEntryPointInWorker("onStoragePermissionRequest", req);
+  delegate->CallEntryPointInWorker("onStoragePermissionRequest", req);
 }
 
 bool WebContextIODelegate::GetUserAgentOverride(const QUrl& url,
@@ -162,16 +162,15 @@ bool WebContextIODelegate::GetUserAgentOverride(const QUrl& url,
 
   bool did_override = false;
 
-  OxideQUserAgentOverrideRequest* req = new OxideQUserAgentOverrideRequest(url);
+  OxideQUserAgentOverrideRequest req(url);
+
+  delegate->CallEntryPointInWorker("onGetUserAgentOverride", &req);
 
   OxideQUserAgentOverrideRequestPrivate* p =
-      OxideQUserAgentOverrideRequestPrivate::get(req);
-  p->did_override = &did_override;
-  p->user_agent = user_agent;
+      OxideQUserAgentOverrideRequestPrivate::get(&req);
+  *user_agent = p->user_agent;
 
-  emit delegate->callEntryPointInWorker("onGetUserAgentOverride", req);
-
-  return did_override;
+  return p->did_override;
 }
 
 } // namespace qquick
@@ -179,18 +178,21 @@ bool WebContextIODelegate::GetUserAgentOverride(const QUrl& url,
 
 using namespace oxide::qquick;
 
+OXIDE_Q_IMPL_PROXY_HANDLE_CONVERTER(OxideQQuickWebContext,
+                                    oxide::qt::WebContextProxyHandle);
+
 OxideQQuickWebContextPrivate::OxideQQuickWebContextPrivate(
     OxideQQuickWebContext* q)
-    : oxide::qt::WebContextAdapter(q),
+    : oxide::qt::WebContextProxyHandle(oxide::qt::WebContextProxy::create(this), q),
       constructed_(false),
       io_(new oxide::qquick::WebContextIODelegate()),
-      network_request_delegate_(NULL),
-      storage_access_permission_delegate_(NULL),
-      user_agent_override_delegate_(NULL),
-      cookie_manager_(NULL) {}
+      network_request_delegate_(nullptr),
+      storage_access_permission_delegate_(nullptr),
+      user_agent_override_delegate_(nullptr),
+      cookie_manager_(nullptr) {}
 
 void OxideQQuickWebContextPrivate::userScriptUpdated() {
-  updateUserScripts();
+  proxy()->updateUserScripts();
 }
 
 void OxideQQuickWebContextPrivate::userScriptWillBeDeleted() {
@@ -199,7 +201,7 @@ void OxideQQuickWebContextPrivate::userScriptWillBeDeleted() {
   OxideQQuickUserScriptPrivate* sender =
       qobject_cast<OxideQQuickUserScriptPrivate *>(q->sender());
   Q_ASSERT(sender);
-  q->removeUserScript(adapterToQObject<OxideQQuickUserScript>(sender));  
+  q->removeUserScript(OxideQQuickUserScriptPrivate::fromProxyHandle(sender));
 }
 
 void OxideQQuickWebContextPrivate::detachUserScriptSignals(
@@ -230,34 +232,38 @@ void OxideQQuickWebContextPrivate::userScript_append(
 
 int OxideQQuickWebContextPrivate::userScript_count(
     QQmlListProperty<OxideQQuickUserScript>* prop) {
-  OxideQQuickWebContextPrivate* cd = OxideQQuickWebContextPrivate::get(
-        static_cast<OxideQQuickWebContext *>(prop->object));
+  oxide::qt::WebContextProxy* p =
+      OxideQQuickWebContextPrivate::get(
+        static_cast<OxideQQuickWebContext*>(prop->object))->proxy();
 
-  return cd->userScripts().size();
+  return p->userScripts().size();
 }
 
 OxideQQuickUserScript* OxideQQuickWebContextPrivate::userScript_at(
     QQmlListProperty<OxideQQuickUserScript>* prop,
     int index) {
-  OxideQQuickWebContextPrivate* cd = OxideQQuickWebContextPrivate::get(
-        static_cast<OxideQQuickWebContext *>(prop->object));
+  oxide::qt::WebContextProxy* p =
+      OxideQQuickWebContextPrivate::get(
+        static_cast<OxideQQuickWebContext*>(prop->object))->proxy();
 
-  if (index >= cd->userScripts().size()) {
-    return NULL;
+  if (index >= p->userScripts().size()) {
+    return nullptr;
   }
 
-  return adapterToQObject<OxideQQuickUserScript>(cd->userScripts().at(index));
+  return OxideQQuickUserScriptPrivate::fromProxyHandle(
+      p->userScripts().at(index));
 }
 
 void OxideQQuickWebContextPrivate::userScript_clear(
     QQmlListProperty<OxideQQuickUserScript>* prop) {
   OxideQQuickWebContext* context =
       static_cast<OxideQQuickWebContext *>(prop->object);
-  OxideQQuickWebContextPrivate* cd = OxideQQuickWebContextPrivate::get(context);
+  oxide::qt::WebContextProxy* p =
+      OxideQQuickWebContextPrivate::get(context)->proxy();
 
-  while (cd->userScripts().size() > 0) {
+  while (p->userScripts().size() > 0) {
     context->removeUserScript(
-        adapterToQObject<OxideQQuickUserScript>(cd->userScripts().at(0)));
+        OxideQQuickUserScriptPrivate::fromProxyHandle(p->userScripts().at(0)));
   }
 }
 
@@ -307,9 +313,7 @@ void OxideQQuickWebContextPrivate::detachedDelegateWorker(
 void OxideQQuickWebContextPrivate::CookiesSet(
     int request_id,
     const QList<QNetworkCookie>& failed_cookies) {
-  Q_Q(OxideQQuickWebContext);
-
-  emit q->cookieManager()->setCookiesResponse(
+  emit cookie_manager_->setCookiesResponse(
       request_id,
       networkCookiesToVariant(failed_cookies));
 }
@@ -317,13 +321,35 @@ void OxideQQuickWebContextPrivate::CookiesSet(
 void OxideQQuickWebContextPrivate::CookiesRetrieved(
       int request_id,
       const QList<QNetworkCookie>& cookies) {
-  Q_Q(OxideQQuickWebContext);
-
-  emit q->cookieManager()->getCookiesResponse(
+  emit cookie_manager_->getCookiesResponse(
       request_id,
       networkCookiesToVariant(cookies));
 }
 
+void OxideQQuickWebContextPrivate::CookiesDeleted(
+    int request_id, int num_deleted) {
+  emit cookie_manager_->deleteCookiesResponse(request_id, num_deleted);
+}
+
+QNetworkAccessManager*
+OxideQQuickWebContextPrivate::GetCustomNetworkAccessManager() {
+  Q_Q(OxideQQuickWebContext);
+
+  QQmlEngine* engine = qmlEngine(q);
+  if (!engine) {
+    return nullptr;
+  }
+
+  return engine->networkAccessManager();
+}
+
+void OxideQQuickWebContextPrivate::DestroyDefault() {
+  Q_Q(OxideQQuickWebContext);
+
+  Q_ASSERT(q == OxideQQuickWebContext::defaultContext(false));
+  delete q;
+}
+
 OxideQQuickWebContextPrivate::~OxideQQuickWebContextPrivate() {}
 
 void OxideQQuickWebContextPrivate::delegateWorkerDestroyed(
@@ -331,13 +357,13 @@ void OxideQQuickWebContextPrivate::delegateWorkerDestroyed(
   Q_Q(OxideQQuickWebContext);
 
   if (worker == q->networkRequestDelegate()) {
-    q->setNetworkRequestDelegate(NULL);
+    q->setNetworkRequestDelegate(nullptr);
   }
   if (worker == q->storageAccessPermissionDelegate()) {
-    q->setStorageAccessPermissionDelegate(NULL);
+    q->setStorageAccessPermissionDelegate(nullptr);
   }
   if (worker == q->userAgentOverrideDelegate()) {
-    q->setUserAgentOverrideDelegate(NULL);
+    q->setUserAgentOverrideDelegate(nullptr);
   }
 }
 
@@ -346,23 +372,26 @@ OxideQQuickWebContextPrivate* OxideQQuickWebContextPrivate::get(
   return context->d_func();
 }
 
-// static
-void OxideQQuickWebContextPrivate::ensureChromiumStarted() {
-  static bool started = false;
-  if (started) {
-    return;
-  }
-  started = true;
-#if defined(ENABLE_COMPOSITING)
-  oxide::qt::WebContextAdapter::setSharedGLContext(
-#if QT_VERSION < QT_VERSION_CHECK(5, 3, 0)
-      QSGContext::sharedOpenGLContext()
-#else
-      QOpenGLContextPrivate::globalShareContext()
-#endif
-  );
-#endif
-  oxide::qt::WebContextAdapter::ensureChromiumStarted();
+bool OxideQQuickWebContextPrivate::isInitialized() const {
+  return proxy()->isInitialized();
+}
+
+int OxideQQuickWebContextPrivate::setCookies(
+    const QUrl& url,
+    const QList<QNetworkCookie>& cookies) {
+  return proxy()->setCookies(url, cookies);
+}
+
+int OxideQQuickWebContextPrivate::getCookies(const QUrl& url) {
+  return proxy()->getCookies(url);
+}
+
+int OxideQQuickWebContextPrivate::getAllCookies() {
+  return proxy()->getAllCookies();
+}
+
+int OxideQQuickWebContextPrivate::deleteAllCookies() {
+  return proxy()->deleteAllCookies();
 }
 
 OxideQQuickWebContext::OxideQQuickWebContext(QObject* parent) :
@@ -370,23 +399,20 @@ OxideQQuickWebContext::OxideQQuickWebContext(QObject* parent) :
     d_ptr(new OxideQQuickWebContextPrivate(this)) {
   Q_D(OxideQQuickWebContext);
 
-  QSharedPointer<oxide::qt::WebContextAdapter::IODelegate> io =
-      qSharedPointerCast<oxide::qt::WebContextAdapter::IODelegate>(d->io_);
-  d->init(io.toWeakRef());
+  QSharedPointer<oxide::qt::WebContextProxyClient::IOClient> io =
+      qSharedPointerCast<oxide::qt::WebContextProxyClient::IOClient>(d->io_);
+  d->proxy()->init(io.toWeakRef());
 
-  OxideQQuickWebContextPrivate::ensureChromiumStarted();
+  oxide::qquick::EnsureChromiumStarted();
 }
 
 OxideQQuickWebContext::~OxideQQuickWebContext() {
   Q_D(OxideQQuickWebContext);
 
-  Q_ASSERT(this != g_default_context);
-
-  emit d->willBeDestroyed();
-
-  for (int i = 0; i < d->userScripts().size(); ++i) {
+  for (int i = 0; i < d->proxy()->userScripts().size(); ++i) {
     d->detachUserScriptSignals(
-        adapterToQObject<OxideQQuickUserScript>(d->userScripts().at(i)));
+        OxideQQuickUserScriptPrivate::fromProxyHandle(
+          d->proxy()->userScripts().at(i)));
   }
 
   // These call back in to us when destroyed, so delete them now in order
@@ -407,39 +433,42 @@ void OxideQQuickWebContext::componentComplete() {
 
 // static
 OxideQQuickWebContext* OxideQQuickWebContext::defaultContext(bool create) {
-  if (g_default_context) {
-    return g_default_context;
+  oxide::qt::WebContextProxyHandle* h =
+      oxide::qt::WebContextProxy::defaultContext();
+  if (h) {
+    return OxideQQuickWebContextPrivate::fromProxyHandle(h);
   }
 
   if (!create) {
-    return NULL;
+    return nullptr;
   }
 
-  g_default_context = new OxideQQuickWebContext();
-  g_default_context->componentComplete();
-  qAddPostRoutine(DestroyDefaultContext);
+  OxideQQuickWebContext* c = new OxideQQuickWebContext();
+  c->componentComplete();
+  QQmlEngine::setObjectOwnership(c, QQmlEngine::CppOwnership);
 
-  QQmlEngine::setObjectOwnership(g_default_context, QQmlEngine::CppOwnership);
+  OxideQQuickWebContextPrivate::get(c)->proxy()->makeDefault();
+  Q_ASSERT(oxide::qt::WebContextProxy::defaultContext());
 
-  return g_default_context;
+  return c;
 }
 
 QString OxideQQuickWebContext::product() const {
   Q_D(const OxideQQuickWebContext);
 
-  return d->product();
+  return d->proxy()->product();
 }
 
 void OxideQQuickWebContext::setProduct(const QString& product) {
   Q_D(OxideQQuickWebContext);
 
-  if (d->product() == product) {
+  if (d->proxy()->product() == product) {
     return;
   }
 
   QString old_user_agent = userAgent();
 
-  d->setProduct(product);
+  d->proxy()->setProduct(product);
   emit productChanged();
 
   if (userAgent() != old_user_agent) {
@@ -450,85 +479,96 @@ void OxideQQuickWebContext::setProduct(const QString& product) {
 QString OxideQQuickWebContext::userAgent() const {
   Q_D(const OxideQQuickWebContext);
 
-  return d->userAgent();
+  return d->proxy()->userAgent();
 }
 
 void OxideQQuickWebContext::setUserAgent(const QString& user_agent) {
   Q_D(OxideQQuickWebContext);
 
-  if (d->userAgent() == user_agent) {
+  if (d->proxy()->userAgent() == user_agent) {
     return;
   }
 
-  d->setUserAgent(user_agent);
+  d->proxy()->setUserAgent(user_agent);
   emit userAgentChanged();
 }
 
 QUrl OxideQQuickWebContext::dataPath() const {
   Q_D(const OxideQQuickWebContext);
 
-  return d->dataPath();
+  return d->proxy()->dataPath();
 }
 
 void OxideQQuickWebContext::setDataPath(const QUrl& data_url) {
   Q_D(OxideQQuickWebContext);
 
-  if (d->isInitialized()) {
-    qWarning() << "Cannot set WebContext.dataPath once the context is in use";
+  if (d->proxy()->isInitialized()) {
+    qWarning() <<
+        "OxideQQuickWebContext: Cannot set dataPath once the context is in use";
     return;
   }
 
-  if (d->dataPath() == data_url) {
+  if (dataPath() == data_url) {
     return;
   }
 
-  d->setDataPath(data_url);
+  if (!data_url.isLocalFile() && !data_url.isEmpty()) {
+    qWarning() << "OxideQQuickWebContext: dataPath only supports local files";
+    return;
+  }
+
+  d->proxy()->setDataPath(data_url);
   emit dataPathChanged();
 }
 
 QUrl OxideQQuickWebContext::cachePath() const {
   Q_D(const OxideQQuickWebContext);
 
-  return d->cachePath();
+  return d->proxy()->cachePath();
 }
 
 void OxideQQuickWebContext::setCachePath(const QUrl& cache_url) {
   Q_D(OxideQQuickWebContext);
 
-  if (d->isInitialized()) {
-    qWarning() << "Cannot set WebContext.cachePath once the context is in use";
+  if (d->proxy()->isInitialized()) {
+    qWarning() << "OxideQQuickWebContext:: Cannot set cachePath once the context is in use";
     return;
   }
 
-  if (d->cachePath() == cache_url) {
+  if (cachePath() == cache_url) {
     return;
   }
 
-  d->setCachePath(cache_url);
+  if (!cache_url.isLocalFile() && !cache_url.isEmpty()) {
+    qWarning() << "OxideQQuickWebContext: cachePath only supports local files";
+    return;
+  }
+
+  d->proxy()->setCachePath(cache_url);
   emit cachePathChanged();
 }
 
 QString OxideQQuickWebContext::acceptLangs() const {
   Q_D(const OxideQQuickWebContext);
 
-  return d->acceptLangs();
+  return d->proxy()->acceptLangs();
 }
 
 void OxideQQuickWebContext::setAcceptLangs(const QString& accept_langs) {
   Q_D(OxideQQuickWebContext);
 
-  if (d->acceptLangs() == accept_langs) {
+  if (acceptLangs() == accept_langs) {
     return;
   }
 
-  d->setAcceptLangs(accept_langs);
+  d->proxy()->setAcceptLangs(accept_langs);
   emit acceptLangsChanged();
 }
 
 QQmlListProperty<OxideQQuickUserScript>
 OxideQQuickWebContext::userScripts() {
   return QQmlListProperty<OxideQQuickUserScript>(
-      this, NULL,
+      this, nullptr,
       OxideQQuickWebContextPrivate::userScript_append,
       OxideQQuickWebContextPrivate::userScript_count,
       OxideQQuickWebContextPrivate::userScript_at,
@@ -546,7 +586,7 @@ void OxideQQuickWebContext::addUserScript(OxideQQuickUserScript* user_script) {
   OxideQQuickUserScriptPrivate* ud =
       OxideQQuickUserScriptPrivate::get(user_script);
 
-  if (!d->userScripts().contains(ud)) {
+  if (!d->proxy()->userScripts().contains(ud)) {
     connect(user_script, SIGNAL(scriptLoaded()),
             this, SLOT(userScriptUpdated()));
     connect(user_script, SIGNAL(scriptPropertyChanged()),
@@ -554,13 +594,13 @@ void OxideQQuickWebContext::addUserScript(OxideQQuickUserScript* user_script) {
     connect(ud, SIGNAL(willBeDeleted()),
             this, SLOT(userScriptWillBeDeleted()));
   } else {
-    d->userScripts().removeOne(ud);
+    d->proxy()->userScripts().removeOne(ud);
   }
 
   if (!user_script->parent()) {
     user_script->setParent(this);
   }
-  d->userScripts().append(ud);
+  d->proxy()->userScripts().append(ud);
 
   emit userScriptsChanged();
 }
@@ -577,16 +617,16 @@ void OxideQQuickWebContext::removeUserScript(
   OxideQQuickUserScriptPrivate* ud =
       OxideQQuickUserScriptPrivate::get(user_script);
 
-  if (!d->userScripts().contains(ud)) {
+  if (!d->proxy()->userScripts().contains(ud)) {
     return;
   }
 
   d->detachUserScriptSignals(user_script);
   if (user_script->parent() == this) {
-    user_script->setParent(NULL);
+    user_script->setParent(nullptr);
   }
 
-  d->userScripts().removeOne(ud);
+  d->proxy()->userScripts().removeOne(ud);
 
   emit userScriptsChanged();
 }
@@ -597,30 +637,30 @@ OxideQQuickWebContext::CookiePolicy OxideQQuickWebContext::cookiePolicy() const
   Q_STATIC_ASSERT(
       CookiePolicyAllowAll ==
       static_cast<CookiePolicy>(
-        oxide::qt::WebContextAdapter::CookiePolicyAllowAll));
+        oxide::qt::WebContextProxy::CookiePolicyAllowAll));
   Q_STATIC_ASSERT(
       CookiePolicyBlockAll ==
       static_cast<CookiePolicy>(
-        oxide::qt::WebContextAdapter::CookiePolicyBlockAll));
+        oxide::qt::WebContextProxy::CookiePolicyBlockAll));
   Q_STATIC_ASSERT(
       CookiePolicyBlockThirdParty ==
       static_cast<CookiePolicy>(
-        oxide::qt::WebContextAdapter::CookiePolicyBlockThirdParty));
+        oxide::qt::WebContextProxy::CookiePolicyBlockThirdParty));
 
-  return static_cast<CookiePolicy>(d->cookiePolicy());
+  return static_cast<CookiePolicy>(d->proxy()->cookiePolicy());
 }
 
 void OxideQQuickWebContext::setCookiePolicy(CookiePolicy policy) {
   Q_D(OxideQQuickWebContext);
 
-  oxide::qt::WebContextAdapter::CookiePolicy p =
-      static_cast<oxide::qt::WebContextAdapter::CookiePolicy>(policy);
+  oxide::qt::WebContextProxy::CookiePolicy p =
+      static_cast<oxide::qt::WebContextProxy::CookiePolicy>(policy);
 
-  if (p == d->cookiePolicy()) {
+  if (policy == cookiePolicy()) {
     return;
   }
 
-  d->setCookiePolicy(p);
+  d->proxy()->setCookiePolicy(p);
 
   emit cookiePolicyChanged();
 }
@@ -632,34 +672,36 @@ OxideQQuickWebContext::sessionCookieMode() const {
   Q_STATIC_ASSERT(
       SessionCookieModeEphemeral ==
       static_cast<SessionCookieMode>(
-        oxide::qt::WebContextAdapter::SessionCookieModeEphemeral));
+        oxide::qt::WebContextProxy::SessionCookieModeEphemeral));
   Q_STATIC_ASSERT(
       SessionCookieModePersistent ==
       static_cast<SessionCookieMode>(
-        oxide::qt::WebContextAdapter::SessionCookieModePersistent));
+        oxide::qt::WebContextProxy::SessionCookieModePersistent));
   Q_STATIC_ASSERT(
       SessionCookieModeRestored ==
       static_cast<SessionCookieMode>(
-        oxide::qt::WebContextAdapter::SessionCookieModeRestored));
-  return static_cast<SessionCookieMode>(d->sessionCookieMode());
+        oxide::qt::WebContextProxy::SessionCookieModeRestored));
+  return static_cast<SessionCookieMode>(d->proxy()->sessionCookieMode());
 }
 
 void OxideQQuickWebContext::setSessionCookieMode(SessionCookieMode mode) {
   Q_D(OxideQQuickWebContext);
 
-  if (d->isInitialized()) {
-    qWarning() << "Cannot set WebContext.sessionCookieMode once the context is in use";
+  if (d->proxy()->isInitialized()) {
+    qWarning() <<
+        "OxideQQuickWebContext::Cannot set sessionCookieMode once the context "
+        "is in use";
     return;
   }
 
-  oxide::qt::WebContextAdapter::SessionCookieMode m =
-      static_cast<oxide::qt::WebContextAdapter::SessionCookieMode>(mode);
+  oxide::qt::WebContextProxy::SessionCookieMode m =
+      static_cast<oxide::qt::WebContextProxy::SessionCookieMode>(mode);
 
-  if (m == d->sessionCookieMode()) {
+  if (mode == sessionCookieMode()) {
     return;
   }
 
-  d->setSessionCookieMode(m);
+  d->proxy()->setSessionCookieMode(m);
 
   emit sessionCookieModeChanged();
 }
@@ -667,17 +709,17 @@ void OxideQQuickWebContext::setSessionCookieMode(SessionCookieMode mode) {
 bool OxideQQuickWebContext::popupBlockerEnabled() const {
   Q_D(const OxideQQuickWebContext);
 
-  return d->popupBlockerEnabled();
+  return d->proxy()->popupBlockerEnabled();
 }
 
 void OxideQQuickWebContext::setPopupBlockerEnabled(bool enabled) {
   Q_D(OxideQQuickWebContext);
 
-  if (d->popupBlockerEnabled() == enabled) {
+  if (popupBlockerEnabled() == enabled) {
     return;
   }
 
-  d->setPopupBlockerEnabled(enabled);
+  d->proxy()->setPopupBlockerEnabled(enabled);
 
   emit popupBlockerEnabledChanged();
 }
@@ -796,17 +838,24 @@ void OxideQQuickWebContext::setUserAgentOverrideDelegate(
 bool OxideQQuickWebContext::devtoolsEnabled() const {
   Q_D(const OxideQQuickWebContext);
 
-  return d->devtoolsEnabled();
+  return d->proxy()->devtoolsEnabled();
 }
 
 void OxideQQuickWebContext::setDevtoolsEnabled(bool enabled) {
   Q_D(OxideQQuickWebContext);
 
-  if (d->devtoolsEnabled() == enabled) {
+  if (d->proxy()->isInitialized()) {
+    qWarning() <<
+        "OxideQQuickWebContext: Cannot set devToolsEnabled once the context "
+        "is in use";
     return;
   }
 
-  d->setDevtoolsEnabled(enabled);
+  if (devtoolsEnabled() == enabled) {
+    return;
+  }
+
+  d->proxy()->setDevtoolsEnabled(enabled);
 
   emit devtoolsEnabledChanged();
 }
@@ -814,21 +863,53 @@ void OxideQQuickWebContext::setDevtoolsEnabled(bool enabled) {
 int OxideQQuickWebContext::devtoolsPort() const {
   Q_D(const OxideQQuickWebContext);
 
-  return d->devtoolsPort();
+  return d->proxy()->devtoolsPort();
 }
 
 void OxideQQuickWebContext::setDevtoolsPort(int port) {
   Q_D(OxideQQuickWebContext);
 
-  if (d->devtoolsPort() == port) {
+  if (d->proxy()->isInitialized()) {
+    qWarning() <<
+        "OxideQQuickWebContext: Cannot set devToolsPort once the context is "
+        "in use";
     return;
   }
 
-  d->setDevtoolsPort(port);
+  if (devtoolsPort() == port) {
+    return;
+  }
+
+  d->proxy()->setDevtoolsPort(port);
 
   emit devtoolsPortChanged();
 }
 
+QString OxideQQuickWebContext::devtoolsBindIp() const {
+  Q_D(const OxideQQuickWebContext);
+
+  return d->proxy()->devtoolsBindIp();
+}
+
+void OxideQQuickWebContext::setDevtoolsBindIp(const QString& bindIp) {
+  Q_D(OxideQQuickWebContext);
+
+  if (d->proxy()->isInitialized()) {
+    qWarning() <<
+        "OxideQQuickWebContext: Cannot set devToolsBindIp once the context is "
+        "in use";
+    return;
+  }
+
+  if (devtoolsBindIp() == bindIp) {
+    return;
+  }
+
+  d->proxy()->setDevtoolsBindIp(bindIp);
+
+  emit devtoolsBindIpChanged();
+}
+
 OxideQQuickCookieManager*
 OxideQQuickWebContext::cookieManager() const {
   Q_D(const OxideQQuickWebContext);
@@ -843,22 +924,83 @@ OxideQQuickWebContext::cookieManager() const {
   return d->cookie_manager_;
 }
 
-QString OxideQQuickWebContext::devtoolsBindIp() const {
+QStringList OxideQQuickWebContext::hostMappingRules() const {
   Q_D(const OxideQQuickWebContext);
 
-  return d->devtoolsBindIp();
+  return d->proxy()->hostMappingRules();
 }
 
-void OxideQQuickWebContext::setDevtoolsBindIp(const QString& bindIp) {
+void OxideQQuickWebContext::setHostMappingRules(const QStringList& rules) {
   Q_D(OxideQQuickWebContext);
 
-  if (d->devtoolsBindIp() == bindIp) {
+  if (d->proxy()->isInitialized()) {
+    qWarning() <<
+        "OxideQQuickWebContext: Cannot set hostMappingRules once the context "
+        "is in use";
+    return; 
+  }
+
+  if (rules == hostMappingRules()) {
     return;
   }
 
-  d->setDevtoolsBindIp(bindIp);
+  d->proxy()->setHostMappingRules(rules);
 
-  emit devtoolsBindIpChanged();
+  emit hostMappingRulesChanged();
+}
+
+QStringList OxideQQuickWebContext::allowedExtraUrlSchemes() const {
+  Q_D(const OxideQQuickWebContext);
+
+  return d->allowed_extra_url_schemes_;
+}
+
+void OxideQQuickWebContext::setAllowedExtraUrlSchemes(
+    const QStringList& schemes) {
+  Q_D(OxideQQuickWebContext);
+
+  d->allowed_extra_url_schemes_ = schemes;
+  d->proxy()->setAllowedExtraUrlSchemes(schemes);
+
+  emit allowedExtraUrlSchemesChanged();
+}
+
+int OxideQQuickWebContext::maxCacheSizeHint() const {
+  Q_D(const OxideQQuickWebContext);
+
+  return d->proxy()->maxCacheSizeHint();
+}
+
+void OxideQQuickWebContext::setMaxCacheSizeHint(int size) {
+  Q_D(OxideQQuickWebContext);
+
+  if (d->proxy()->isInitialized()) {
+    qWarning() <<
+        "OxideQQuickWebContext: Cannot set maxCacheSizeHint once the context "
+        "is in use";
+    return;
+  }
+
+  if (size < 0) {
+    qWarning() <<
+        "OxideQQuickWebContext: maxCacheSizeHint cannot have a negative value";
+    return;
+  }
+
+  static int upper_limit = std::numeric_limits<int>::max() / (1024 * 1024);
+  if (size > upper_limit) {
+    // To avoid integer overflow.
+    qWarning() << "OxideQQuickWebContext: maxCacheSizeHint cannot exceed "
+               << upper_limit << "MB";
+    return;
+  }
+
+  if (maxCacheSizeHint() == size) {
+    return;
+  }
+
+  d->proxy()->setMaxCacheSizeHint(size);
+  emit maxCacheSizeHintChanged();
 }
 
 #include "moc_oxideqquickwebcontext_p.cpp"
