@@ -1,5 +1,5 @@
 // vim:expandtab:shiftwidth=2:tabstop=2:
-// Copyright (C) 2013 Canonical Ltd.
+// Copyright (C) 2013-2015 Canonical Ltd.
 
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Lesser General Public
@@ -19,24 +19,28 @@
 
 #include <algorithm>
 #include <libintl.h>
+#include <limits>
 #include <vector>
 
+#include "base/files/file_enumerator.h"
 #include "base/files/file_path.h"
+#include "base/files/file_util.h"
 #include "base/lazy_instance.h"
 #include "base/logging.h"
 #include "base/memory/weak_ptr.h"
 #include "base/strings/stringprintf.h"
 #include "base/supports_user_data.h"
 #include "base/synchronization/lock.h"
+#include "base/threading/thread_restrictions.h"
 #include "base/threading/worker_pool.h"
 #include "content/browser/loader/resource_dispatcher_host_impl.h"
 #include "content/public/browser/browser_thread.h"
-#include "content/public/browser/cookie_crypto_delegate.h"
 #include "content/public/browser/devtools_http_handler.h"
 #include "content/public/browser/render_process_host.h"
 #include "content/public/browser/resource_context.h"
 #include "content/public/browser/storage_partition.h"
 #include "content/public/common/user_agent.h"
+#include "net/base/host_mapping_rules.h"
 #include "net/base/net_errors.h"
 #include "net/base/net_util.h"
 #include "net/cookies/cookie_monster.h"
@@ -46,7 +50,7 @@
 #include "net/http/http_server_properties_impl.h"
 #include "net/http/transport_security_persister.h"
 #include "net/http/transport_security_state.h"
-#include "net/socket/tcp_listen_socket.h"
+#include "net/socket/tcp_server_socket.h"
 #include "net/ssl/channel_id_service.h"
 #include "net/ssl/channel_id_store.h"
 #include "net/ssl/default_channel_id_store.h"
@@ -68,9 +72,11 @@
 #include "oxide_http_user_agent_settings.h"
 #include "oxide_io_thread.h"
 #include "oxide_network_delegate.h"
+#include "oxide_permission_manager.h"
 #include "oxide_ssl_config_service.h"
 #include "oxide_ssl_host_state_delegate.h"
 #include "oxide_url_request_context.h"
+#include "oxide_url_request_delegated_job_factory.h"
 #include "oxide_user_script_master.h"
 
 namespace oxide {
@@ -79,7 +85,11 @@ namespace {
 
 base::LazyInstance<std::vector<BrowserContext *> > g_all_contexts;
 
+// Cache was used for the default blockfile backend (CACHE_BACKEND_BLOCKFILE),
+// Cache2 is used since the switch to the simple backend (CACHE_BACKEND_SIMPLE).
 const base::FilePath::CharType kCacheDirname[] = FILE_PATH_LITERAL("Cache");
+const base::FilePath::CharType kCacheDirname2[] = FILE_PATH_LITERAL("Cache2");
+
 const base::FilePath::CharType kCookiesFilename[] =
     FILE_PATH_LITERAL("cookies.sqlite");
 
@@ -90,7 +100,66 @@ const char kFtpScheme[] = "ftp";
 const char kBrowserContextKey[] = "oxide_browser_context_data";
 
 const char kDefaultAcceptLanguage[] = "en-us,en";
+
 const char kDevtoolsDefaultServerIp[] = "127.0.0.1";
+const int kBackLog = 1;
+
+void CleanupOldCacheDir(const base::FilePath& path) {
+  if (!base::DirectoryExists(path)) {
+    return;
+  }
+
+  base::FileEnumerator traversal(
+      path, false,
+      base::FileEnumerator::FILES | base::FileEnumerator::DIRECTORIES);
+  for (base::FilePath current = traversal.Next(); !current.empty();
+       current = traversal.Next()) {
+    if (traversal.GetInfo().IsDirectory()) {
+      return;
+    }
+
+    base::FilePath::StringType name = traversal.GetInfo().GetName().value();
+    if (name != FILE_PATH_LITERAL("index") &&
+        name.compare(0, 5, FILE_PATH_LITERAL("data_")) != 0 &&
+        name.compare(0, 2, FILE_PATH_LITERAL("f_")) != 0) {
+      return;
+    }
+  }
+
+  base::DeleteFile(path, true);
+}
+
+} // namespace
+
+class TCPServerSocketFactory :
+    public content::DevToolsHttpHandler::ServerSocketFactory {
+ public:
+  TCPServerSocketFactory(const std::string& address, int port)
+      : address_(address),
+        port_(port) {}
+
+ private:
+  scoped_ptr<net::ServerSocket> CreateForHttpServer() override {
+    scoped_ptr<net::TCPServerSocket> socket(
+        new net::TCPServerSocket(nullptr, net::NetLog::Source()));
+    if (socket->ListenWithAddressAndPort(address_, port_, kBackLog) != net::OK) {
+      return scoped_ptr<net::ServerSocket>();
+    }
+
+    return socket.Pass();
+  }
+
+  scoped_ptr<net::ServerSocket> CreateForTethering(
+      std::string* out_name) override {
+    // Not supported
+    return scoped_ptr<net::ServerSocket>();
+  }
+
+  std::string address_;
+  int port_;
+
+  DISALLOW_COPY_AND_ASSIGN(TCPServerSocketFactory);
+};
 
 class ResourceContextData : public base::SupportsUserData::Data {
  public:
@@ -104,7 +173,7 @@ class ResourceContextData : public base::SupportsUserData::Data {
   BrowserContextIOData* context_;
 };
 
-class MainURLRequestContextGetter FINAL : public URLRequestContextGetter {
+class MainURLRequestContextGetter final : public URLRequestContextGetter {
  public:
   MainURLRequestContextGetter(
       BrowserContextIOData* context,
@@ -115,14 +184,14 @@ class MainURLRequestContextGetter FINAL : public URLRequestContextGetter {
     std::swap(protocol_handlers_, *protocol_handlers);
   }
 
-  net::URLRequestContext* GetURLRequestContext() FINAL {
+  net::URLRequestContext* GetURLRequestContext() final {
     DCHECK_CURRENTLY_ON(content::BrowserThread::IO);
 
     if (!url_request_context_) {
       DCHECK(context_);
       url_request_context_ = context_->CreateMainRequestContext(
           protocol_handlers_, request_interceptors_.Pass())->AsWeakPtr();
-      context_ = NULL;
+      context_ = nullptr;
     }
 
     return url_request_context_.get();
@@ -136,30 +205,20 @@ class MainURLRequestContextGetter FINAL : public URLRequestContextGetter {
   base::WeakPtr<URLRequestContext> url_request_context_;
 };
 
-} // namespace
-
-class ResourceContext FINAL : public content::ResourceContext {
+class ResourceContext final : public content::ResourceContext {
  public:
   ResourceContext() :
-      request_context_(NULL) {}
+      request_context_(nullptr) {}
 
-  net::HostResolver* GetHostResolver() FINAL {
+  net::HostResolver* GetHostResolver() final {
     return IOThread::instance()->globals()->host_resolver();
   }
 
-  net::URLRequestContext* GetRequestContext() FINAL {
+  net::URLRequestContext* GetRequestContext() final {
     CHECK(request_context_);
     return request_context_;
   }
 
-  bool AllowMicAccess(const GURL& origin) FINAL {
-    return false;
-  }
-
-  bool AllowCameraAccess(const GURL& origin) FINAL {
-    return false;
-  }
-
  private:
   friend class BrowserContextIOData;
 
@@ -176,7 +235,6 @@ struct BrowserContextSharedData {
         product(base::StringPrintf("Chrome/%s", CHROME_VERSION_STRING)),
         user_agent_string_is_default(true),
         user_script_master(context),
-        devtools_http_handler(NULL),
         devtools_enabled(params.devtools_enabled),
         devtools_port(params.devtools_port),
         devtools_ip(params.devtools_ip) {}
@@ -188,7 +246,7 @@ struct BrowserContextSharedData {
   bool user_agent_string_is_default;
   UserScriptMaster user_script_master;
 
-  content::DevToolsHttpHandler* devtools_http_handler;
+  scoped_ptr<content::DevToolsHttpHandler> devtools_http_handler;
   bool devtools_enabled;
   int devtools_port;
   std::string devtools_ip;
@@ -198,9 +256,11 @@ struct BrowserContextSharedIOData {
   BrowserContextSharedIOData(const BrowserContext::Params& params)
       : path(params.path),
         cache_path(params.cache_path),
+        max_cache_size_hint(params.max_cache_size_hint),
         cookie_policy(net::StaticCookiePolicy::ALLOW_ALL_COOKIES),
         session_cookie_mode(params.session_cookie_mode),
-        popup_blocker_enabled(true) {
+        popup_blocker_enabled(true),
+        host_mapping_rules(params.host_mapping_rules) {
 
    accept_langs = dgettext(OXIDE_GETTEXT_DOMAIN, "AcceptLanguage");
    if (accept_langs == "AcceptLanguage") {
@@ -212,6 +272,7 @@ struct BrowserContextSharedIOData {
 
   base::FilePath path;
   base::FilePath cache_path;
+  int max_cache_size_hint;
 
   std::string user_agent_string;
   std::string accept_langs;
@@ -220,6 +281,8 @@ struct BrowserContextSharedIOData {
   content::CookieStoreConfig::SessionCookieMode session_cookie_mode;
   bool popup_blocker_enabled;
 
+  std::vector<std::string> host_mapping_rules;
+
   scoped_refptr<BrowserContextDelegate> delegate;
 };
 
@@ -243,21 +306,21 @@ class BrowserContextIODataImpl : public BrowserContextIOData {
   BrowserContextIODataImpl(const BrowserContext::Params& params)
       : data_(params) {}
 
-  BrowserContextSharedIOData& GetSharedData() FINAL {
+  BrowserContextSharedIOData& GetSharedData() final {
     return data_;
   }
-  const BrowserContextSharedIOData& GetSharedData() const FINAL {
+  const BrowserContextSharedIOData& GetSharedData() const final {
     return data_;
   }
 
  private:
-  content::CookieStoreConfig::SessionCookieMode GetSessionCookieMode() const FINAL {
+  content::CookieStoreConfig::SessionCookieMode GetSessionCookieMode() const final {
     return GetPath().empty() ?
         content::CookieStoreConfig::EPHEMERAL_SESSION_COOKIES :
         data_.session_cookie_mode;
   }
  
-  bool IsOffTheRecord() const FINAL {
+  bool IsOffTheRecord() const final {
     return false;
   }
 
@@ -270,18 +333,18 @@ class OTRBrowserContextIODataImpl : public BrowserContextIOData {
       : original_io_data_(original) {}
 
  private:
-  content::CookieStoreConfig::SessionCookieMode GetSessionCookieMode() const FINAL {
+  content::CookieStoreConfig::SessionCookieMode GetSessionCookieMode() const final {
     return content::CookieStoreConfig::EPHEMERAL_SESSION_COOKIES;
   }
 
-  bool IsOffTheRecord() const FINAL {
+  bool IsOffTheRecord() const final {
     return true;
   }
 
-  BrowserContextSharedIOData& GetSharedData() FINAL {
+  BrowserContextSharedIOData& GetSharedData() final {
     return original_io_data_->GetSharedData();
   }
-  const BrowserContextSharedIOData& GetSharedData() const FINAL {
+  const BrowserContextSharedIOData& GetSharedData() const final {
     return original_io_data_->GetSharedData();
   }
 
@@ -289,7 +352,7 @@ class OTRBrowserContextIODataImpl : public BrowserContextIOData {
 };
 
 void BrowserContextIOData::Init() {
-  DCHECK(!cookie_store_);
+  DCHECK(!cookie_store_.get());
 
   base::FilePath cookie_path;
   if (!IsOffTheRecord() && !GetPath().empty()) {
@@ -298,7 +361,7 @@ void BrowserContextIOData::Init() {
   cookie_store_ = content::CreateCookieStore(
       content::CookieStoreConfig(cookie_path,
                                  GetSessionCookieMode(),
-                                 NULL, NULL));
+                                 nullptr, nullptr));
 }
 
 BrowserContextIOData::BrowserContextIOData() :
@@ -347,6 +410,15 @@ base::FilePath BrowserContextIOData::GetCachePath() const {
   return data.cache_path;
 }
 
+int BrowserContextIOData::GetMaxCacheSizeHint() const {
+  int max_cache_size_hint = GetSharedData().max_cache_size_hint;
+  // max_cache_size_hint is expressed in MB, let’s check that
+  // converting it to bytes won’t trigger an integer overflow
+  static int upper_limit = std::numeric_limits<int>::max() / (1024 * 1024);
+  DCHECK_LE(max_cache_size_hint, upper_limit);
+  return max_cache_size_hint;
+}
+
 std::string BrowserContextIOData::GetAcceptLangs() const {
   const BrowserContextSharedIOData& data = GetSharedData();
   base::AutoLock lock(data.lock);
@@ -388,6 +460,15 @@ URLRequestContext* BrowserContextIOData::CreateMainRequestContext(
           content::BrowserThread::FILE),
         IsOffTheRecord()));
 
+  host_mapping_rules_.reset(new net::HostMappingRules());
+  const std::vector<std::string>& host_mapping_rules =
+      GetSharedData().host_mapping_rules;
+  for (std::vector<std::string>::const_iterator it =
+           host_mapping_rules.begin();
+       it != host_mapping_rules.end(); ++it) {
+    host_mapping_rules_->AddRuleFromString(*it);
+  }
+
   main_request_context_.reset(new URLRequestContext());
   URLRequestContext* context = main_request_context_.get();
   net::URLRequestContextStorage* storage = context->storage();
@@ -399,26 +480,33 @@ URLRequestContext* BrowserContextIOData::CreateMainRequestContext(
   // TODO: We want persistent storage here (for non-incognito), but 
   //       SQLiteChannelIDStore is part of chrome
   storage->set_channel_id_service(
-      new net::ChannelIDService(
-          new net::DefaultChannelIDStore(NULL),
-          base::WorkerPool::GetTaskRunner(true)));
+      make_scoped_ptr(new net::ChannelIDService(
+          new net::DefaultChannelIDStore(nullptr),
+          base::WorkerPool::GetTaskRunner(true))));
 
   context->set_http_server_properties(http_server_properties_->GetWeakPtr());
 
-  DCHECK(cookie_store_);
-  storage->set_cookie_store(cookie_store_);
+  DCHECK(cookie_store_.get());
+  storage->set_cookie_store(cookie_store_.get());
 
   context->set_transport_security_state(transport_security_state_.get());
 
-  net::HttpCache::BackendFactory* cache_backend = NULL;
+  // Run-once code that is run when upgrading oxide to
+  // a version that uses the simple cache backend.
+  content::BrowserThread::PostTask(
+      content::BrowserThread::FILE,
+      FROM_HERE,
+      base::Bind(&CleanupOldCacheDir, GetCachePath().Append(kCacheDirname)));
+
+  net::HttpCache::BackendFactory* cache_backend = nullptr;
   if (IsOffTheRecord() || GetCachePath().empty()) {
     cache_backend = net::HttpCache::DefaultBackend::InMemory(0);
   } else {
     cache_backend = new net::HttpCache::DefaultBackend(
           net::DISK_CACHE,
-          net::CACHE_BACKEND_DEFAULT,
-          GetCachePath().Append(kCacheDirname),
-          83886080, // XXX: 80MB - Make this configurable
+          net::CACHE_BACKEND_SIMPLE,
+          GetCachePath().Append(kCacheDirname2),
+          GetMaxCacheSizeHint() * 1024 * 1024, // MB -> bytes
           content::BrowserThread::GetMessageLoopProxyForThread(
               content::BrowserThread::CACHE));
   }
@@ -437,9 +525,14 @@ URLRequestContext* BrowserContextIOData::CreateMainRequestContext(
   session_params.http_server_properties =
       context->http_server_properties();
   session_params.net_log = context->net_log();
+  session_params.host_mapping_rules = host_mapping_rules_.get();
 
-  storage->set_http_transaction_factory(
-      new net::HttpCache(session_params, cache_backend));
+  {
+    // Calls QuickStreamFactory constructor which uses base::CPU
+    base::ThreadRestrictions::ScopedAllowIO allow_io;
+    storage->set_http_transaction_factory(
+        new net::HttpCache(session_params, cache_backend));
+  }
 
   scoped_ptr<net::URLRequestJobFactoryImpl> job_factory(
       new net::URLRequestJobFactoryImpl());
@@ -470,7 +563,8 @@ URLRequestContext* BrowserContextIOData::CreateMainRequestContext(
   DCHECK(set_protocol);
 
   scoped_ptr<net::URLRequestJobFactory> top_job_factory(
-      job_factory.PassAs<net::URLRequestJobFactory>());
+      new URLRequestDelegatedJobFactory(job_factory.Pass(),
+                                        this));
 
   for (content::URLRequestInterceptorScopedVector::reverse_iterator it =
           request_interceptors.rbegin();
@@ -495,7 +589,7 @@ bool BrowserContextIOData::CanAccessCookies(const GURL& url,
                                             const GURL& first_party_url,
                                             bool write) {
   scoped_refptr<BrowserContextDelegate> delegate(GetDelegate());
-  if (delegate) {
+  if (delegate.get()) {
     StoragePermission res =
         delegate->CanAccessStorage(url, first_party_url, write,
                                    STORAGE_TYPE_COOKIES);
@@ -524,13 +618,15 @@ class OTRBrowserContextImpl : public BrowserContext {
 
   virtual ~OTRBrowserContextImpl() {}
 
-  BrowserContext* GetOffTheRecordContext() FINAL {
+  BrowserContext* GetOffTheRecordContext() final {
     return this;
   }
-  BrowserContext* GetOriginalContext() FINAL;
+  BrowserContext* GetOriginalContext() final;
 
-  BrowserContextSharedData& GetSharedData() FINAL;
-  const BrowserContextSharedData& GetSharedData() const FINAL;
+  BrowserContextSharedData& GetSharedData() final;
+  const BrowserContextSharedData& GetSharedData() const final;
+
+  bool HasOffTheRecordContext() const final { return true; }
 
   BrowserContextImpl* original_context_;
 };
@@ -539,36 +635,26 @@ class BrowserContextImpl : public BrowserContext {
  public:
   BrowserContextImpl(const BrowserContext::Params& params);
 
-  BrowserContextSharedData& GetSharedData() FINAL {
+  BrowserContextSharedData& GetSharedData() final {
     return data_;
   }
-  const BrowserContextSharedData& GetSharedData() const FINAL {
+  const BrowserContextSharedData& GetSharedData() const final {
     return data_;
   }
 
  private:
-  virtual ~BrowserContextImpl() {
-    if (otr_context_) {
-      delete otr_context_;
-      otr_context_ = NULL;
-    }
-  }
+  virtual ~BrowserContextImpl();
 
-  BrowserContext* GetOffTheRecordContext() FINAL {
-    if (!otr_context_) {
-      otr_context_ =
-          new OTRBrowserContextImpl(
-            this,
-            static_cast<BrowserContextIODataImpl *>(io_data()));
-    }
+  BrowserContext* GetOffTheRecordContext() final;
 
-    return otr_context_;
-  }
-
-  BrowserContext* GetOriginalContext() FINAL {
+  BrowserContext* GetOriginalContext() final {
     return this;
   }
 
+  bool HasOffTheRecordContext() const final {
+    return otr_context_ != nullptr;
+  }
+
   BrowserContextSharedData data_;
   OTRBrowserContextImpl* otr_context_;
 };
@@ -592,32 +678,64 @@ OTRBrowserContextImpl::OTRBrowserContextImpl(
     : BrowserContext(new OTRBrowserContextIODataImpl(original_io_data)),
       original_context_(original) {}
 
+BrowserContextImpl::~BrowserContextImpl() {
+  if (otr_context_) {
+    delete otr_context_;
+    otr_context_ = nullptr;
+  }
+}
+
+BrowserContext* BrowserContextImpl::GetOffTheRecordContext() {
+  if (!otr_context_) {
+    otr_context_ = new OTRBrowserContextImpl(
+        this,
+        static_cast<BrowserContextIODataImpl *>(io_data()));
+  }
+
+  return otr_context_;
+}
+
 BrowserContextImpl::BrowserContextImpl(const BrowserContext::Params& params)
     : BrowserContext(new BrowserContextIODataImpl(params)),
       data_(this, params),
-      otr_context_(NULL) {
+      otr_context_(nullptr) {
   io_data()->GetSharedData().user_agent_string =
       content::BuildUserAgentFromProduct(data_.product);
 
   if (data_.devtools_enabled &&
       data_.devtools_port < 65535 &&
       data_.devtools_port > 1024) {
-    net::IPAddressNumber ipnumber;
+    net::IPAddressNumber unused;
     std::string ip =
-        net::ParseIPLiteralToNumber(data_.devtools_ip, &ipnumber) ?
+        net::ParseIPLiteralToNumber(data_.devtools_ip, &unused) ?
           data_.devtools_ip : kDevtoolsDefaultServerIp;
 
-    data_.devtools_http_handler = content::DevToolsHttpHandler::Start(
-        new net::TCPListenSocketFactory(ip, data_.devtools_port),
-        std::string(),
-        new DevtoolsHttpHandlerDelegate(ip, data_.devtools_port, this),
-        base::FilePath());
+    scoped_ptr<content::DevToolsHttpHandler::ServerSocketFactory> factory(
+        new TCPServerSocketFactory(ip, data_.devtools_port));
+    data_.devtools_http_handler.reset(
+        content::DevToolsHttpHandler::Start(factory.Pass(),
+                                            std::string(),
+                                            new DevtoolsHttpHandlerDelegate(),
+                                            base::FilePath()));
   }
+
+  if (!GetPath().empty()) {
+    base::FilePath gpu_cache = GetPath().Append(FILE_PATH_LITERAL("GPUCache"));
+    content::BrowserThread::PostTask(
+        content::BrowserThread::FILE,
+        FROM_HERE,
+        base::Bind(&CleanupOldCacheDir, gpu_cache));
+  }
+}
+
+scoped_ptr<content::ZoomLevelDelegate> BrowserContext::CreateZoomLevelDelegate(
+    const base::FilePath& partition_path) {
+  return nullptr;
 }
 
 net::URLRequestContextGetter* BrowserContext::GetRequestContext() {
   DCHECK(CalledOnValidThread());
-  return GetStoragePartition(this, NULL)->GetURLRequestContext();
+  return GetStoragePartition(this, nullptr)->GetURLRequestContext();
 }
 
 net::URLRequestContextGetter*
@@ -653,24 +771,24 @@ BrowserContext::GetMediaRequestContextForStoragePartition(
   // ContentBrowserClient::GetStoragePartitionConfigForSite(), so it's a
   // bug to hit this
   NOTREACHED() << "Invalid request for request context for storage partition";
-  return NULL;
+  return nullptr;
 }
 
 content::DownloadManagerDelegate*
     BrowserContext::GetDownloadManagerDelegate() {
-  return NULL;
+  return nullptr;
 }
 
 content::BrowserPluginGuestManager* BrowserContext::GetGuestManager() {
-  return NULL;
+  return nullptr;
 }
 
-quota::SpecialStoragePolicy* BrowserContext::GetSpecialStoragePolicy() {
-  return NULL;
+storage::SpecialStoragePolicy* BrowserContext::GetSpecialStoragePolicy() {
+  return nullptr;
 }
 
 content::PushMessagingService* BrowserContext::GetPushMessagingService() {
-  return NULL;
+  return nullptr;
 }
 
 content::SSLHostStateDelegate* BrowserContext::GetSSLHostStateDelegate() {
@@ -681,6 +799,14 @@ content::SSLHostStateDelegate* BrowserContext::GetSSLHostStateDelegate() {
   return ssl_host_state_delegate_.get();
 }
 
+content::PermissionManager* BrowserContext::GetPermissionManager() {
+  if (!permission_manager_) {
+    permission_manager_.reset(new PermissionManager());
+  }
+
+  return permission_manager_.get();
+}
+
 void BrowserContext::AddObserver(BrowserContextObserver* observer) {
   DCHECK(CalledOnValidThread());
   observers_.AddObserver(observer);
@@ -698,7 +824,6 @@ void BrowserContext::Delete(const BrowserContext* context) {
 
 BrowserContext::BrowserContext(BrowserContextIOData* io_data) :
     io_data_(io_data) {
-  DCHECK(CalledOnValidThread());
   CHECK(BrowserProcessMain::GetInstance()->IsRunning()) <<
       "The main browser process components must be started before " <<
       "creating a context";
@@ -730,7 +855,7 @@ BrowserContext::~BrowserContext() {
   // Schedule io_data_ to be destroyed on the IO thread
   content::BrowserThread::DeleteSoon(content::BrowserThread::IO,
                                      FROM_HERE, io_data_);
-  io_data_ = NULL;
+  io_data_ = nullptr;
 }
 
 // static
@@ -768,19 +893,19 @@ net::URLRequestContextGetter* BrowserContext::CreateRequestContext(
     content::ProtocolHandlerMap* protocol_handlers,
     content::URLRequestInterceptorScopedVector request_interceptors) {
   DCHECK(CalledOnValidThread());
-  DCHECK(!main_request_context_getter_);
+  DCHECK(!main_request_context_getter_.get());
 
   main_request_context_getter_ =
       new MainURLRequestContextGetter(io_data(),
                                       protocol_handlers,
                                       request_interceptors.Pass());
 
-  return main_request_context_getter_;
+  return main_request_context_getter_.get();
 }
 
 BrowserContextDelegate* BrowserContext::GetDelegate() const {
   DCHECK(CalledOnValidThread());
-  return io_data()->GetDelegate();
+  return io_data()->GetDelegate().get();
 }
 
 void BrowserContext::SetDelegate(BrowserContextDelegate* delegate) {
@@ -799,7 +924,8 @@ bool BrowserContext::IsOffTheRecord() const {
 bool BrowserContext::IsSameContext(BrowserContext* other) const {
   DCHECK(CalledOnValidThread());
   return other->GetOriginalContext() == this ||
-         other->GetOffTheRecordContext() == this;
+         (other->HasOffTheRecordContext() &&
+          other->GetOffTheRecordContext() == this);
 }
 
 base::FilePath BrowserContext::GetPath() const {
@@ -812,6 +938,11 @@ base::FilePath BrowserContext::GetCachePath() const {
   return io_data()->GetCachePath();
 }
 
+int BrowserContext::GetMaxCacheSizeHint() const {
+  DCHECK(CalledOnValidThread());
+  return io_data()->GetMaxCacheSizeHint();
+}
+
 std::string BrowserContext::GetAcceptLangs() const {
   DCHECK(CalledOnValidThread());
   return io_data()->GetAcceptLangs();
@@ -901,9 +1032,11 @@ void BrowserContext::SetIsPopupBlockerEnabled(bool enabled) {
   FOR_EACH_OBSERVER(BrowserContextObserver,
                     GetOriginalContext()->observers_,
                     NotifyPopupBlockerEnabledChanged());
-  FOR_EACH_OBSERVER(BrowserContextObserver,
-                    GetOffTheRecordContext()->observers_,
-                    NotifyPopupBlockerEnabledChanged());
+  if (HasOffTheRecordContext()) {
+    FOR_EACH_OBSERVER(BrowserContextObserver,
+                      GetOffTheRecordContext()->observers_,
+                      NotifyPopupBlockerEnabledChanged());
+  }
 }
 
 bool BrowserContext::GetDevtoolsEnabled() const {
@@ -921,6 +1054,11 @@ std::string BrowserContext::GetDevtoolsBindIp() const {
   return GetSharedData().devtools_ip;
 }
 
+const std::vector<std::string>& BrowserContext::GetHostMappingRules() const {
+  DCHECK(CalledOnValidThread());
+  return io_data()->GetSharedData().host_mapping_rules;
+}
+
 UserScriptMaster& BrowserContext::UserScriptManager() {
   DCHECK(CalledOnValidThread());
   return GetSharedData().user_script_master;
