@@ -25,5 +25,12 @@ const char kFormFactorDesktop[] = "desktop";
 const char kFormFactorTablet[] = "tablet";
 const char kFormFactorPhone[] = "phone";
 const char kIncognito[] = "incognito";
+const char kEnableMediaHubAudio[] = "enable-media-hub-audio";
+const char kMediaHubFixedSessionDomains[] = "media-hub-fixed-session-domains";
 
 } // namespace switches
+
+namespace oxide {
+const int kMainWorldId = 0;
+const char kMainWorldContextUrl[] = "oxide://main-world";
+} // namespace oxide
