@@ -26,7 +26,14 @@ extern const char kFormFactorDesktop[];
 extern const char kFormFactorTablet[];
 extern const char kFormFactorPhone[];
 extern const char kIncognito[];
+extern const char kEnableMediaHubAudio[];
+extern const char kMediaHubFixedSessionDomains[];
 
 } // namespace switches
 
+namespace oxide {
+extern const int kMainWorldId;
+extern const char kMainWorldContextUrl[];
+}
+
 #endif // _OXIDE_SHARED_COMMON_CONSTANTS_H_
